package com.persagy.dmp.admin.service;

import com.persagy.common.security.MD5Utils;
import com.persagy.common.web.MapResponse;
import com.persagy.dmp.admin.config.shiro.ShiroContants;
import com.persagy.dmp.admin.config.shiro.UserModel;
import com.persagy.dmp.admin.config.web.UserUtils;
import com.persagy.dmp.admin.entity.AdminUser;
import com.persagy.dmp.admin.entity.QAdminUser;
import com.persagy.dmp.admin.repository.AdminPermissionRepository;
import com.persagy.dmp.admin.repository.AdminUserRepository;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

/**
 * @author: yaoll
 * @date: 2020-12-29
 * @verison: 1.0
 */
@Service
public class AdminService {

	@Autowired
	private AdminUserRepository userRepository;

	@Autowired
	private AdminPermissionRepository permissionRepository;

	@Autowired
	private StringRedisTemplate redis;

	public MapResponse login(UserModel userModel) {
		MapResponse resp = new MapResponse();

		String username = userModel.getUsername();
		String password = userModel.getPassword();
		if (username == null || password == null) {
			resp.setFail("username and password is required");
			return resp;
		}

		Optional<AdminUser> one = userRepository.findOne(QAdminUser.adminUser.username.eq(username));
		if (!one.isPresent()) {
			resp.setFail("username or password error!");
			return resp;
		}
		AdminUser user = one.get();
		if (user.getStatus() != 1) {
			resp.setFail("user is invalid!");
			return resp;
		}

		if (!MD5Utils.getSign(user.getUsername() + password).equalsIgnoreCase(user.getPassword())) {
			resp.setFail("username or password error!");
			return resp;
		}

		String token = UUID.randomUUID().toString().replace("-", "");

		redis.opsForValue().set(ShiroContants.KEY_TOKEN_USER + token, user.getId() + "");
		redis.opsForValue().set(ShiroContants.KEY_USER_TOKEN + user.getId(), token);

		Subject subject = SecurityUtils.getSubject();
		UsernamePasswordToken shiroToken = new UsernamePasswordToken(user.getUsername(), token);
		subject.login(shiroToken);

		resp.add("token", token);
		return resp;
	}

	public MapResponse logout() {
		MapResponse resp = new MapResponse();
		Integer userId = UserUtils.currentUserId();
		if (userId == null) {
			return resp;
		}

		String token = redis.opsForValue().get(ShiroContants.KEY_USER_TOKEN + userId);
		if (token != null) {
			redis.delete(ShiroContants.KEY_TOKEN_USER + token);
		}
		redis.delete(ShiroContants.KEY_USER_TOKEN + userId);

		Subject subject = SecurityUtils.getSubject();
		if (subject.isAuthenticated()) {
			subject.logout();
		}
		return resp;
	}

	public MapResponse userInfo() {
		MapResponse resp = new MapResponse();
		Integer userId = UserUtils.currentUserId();
		Optional<AdminUser> one = userRepository.findOne(QAdminUser.adminUser.id.eq(userId));
		if (!one.isPresent()) {
			resp.setFail("username or password error!");
			return resp;
		}
		AdminUser user = one.get();
		if (user.getStatus() != 1) {
			resp.setFail("user is invalid!");
			return resp;
		}

		resp.add("userId", user.getId());
		resp.add("username", user.getUsername());
		resp.add("realname", user.getRealname());
		resp.add("projects", user.getProjects());

		List<String> permissions = permissionRepository.queryForUser(userId);
		resp.add("permissions", permissions);


		return resp;
	}

	public static void main(String[] args) {
		System.out.println(MD5Utils.getSign("123456"));
	}
}

