package com.persagy.dmp.rwd.edit.service;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.config.web.UserUtils;
import com.persagy.dmp.rwd.edit.entity.QRwdeditRefType;
import com.persagy.dmp.rwd.edit.entity.RwdeditRefType;
import com.persagy.dmp.rwd.edit.model.RwdeditRefTypeModel;
import com.persagy.dmp.rwd.edit.repository.RwdeditRefTypeRepository;
import com.querydsl.core.types.dsl.BooleanExpression;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
public class RwdeditRefTypeService {

    @Autowired
    private CriteriaUtils criteriaUtils;

    @Autowired
    RwdeditRefTypeRepository repository;

    private List<BooleanExpression> parse(ObjectNode object) {
        List<BooleanExpression> list = new LinkedList<>();
        QRwdeditRefType qt = QRwdeditRefType.rwdeditRefType;
        list.addAll(CriteriaUtils.parse(qt.refKey, object.get("refKey")));
        list.addAll(CriteriaUtils.parse(qt.createUser, object.get("createUser")));
        list.addAll(CriteriaUtils.parse(qt.updateUser, object.get("updateUser")));
        return list;
    }

    public PagedResponse<RwdeditRefTypeModel> query(JacksonCriteria criteria) {
        PagedResponse<RwdeditRefType> resp = criteriaUtils.query(QRwdeditRefType.rwdeditRefType, this::parse, criteria);
        PagedResponse<RwdeditRefTypeModel> result = new PagedResponse<>();
        result.setCount(resp.getCount());
        List<RwdeditRefType> dataList = resp.getData();
        Set<String> withColumns = criteria.getWithColumns();
        if (dataList != null && dataList.size() > 0) {
            List<RwdeditRefTypeModel> collect = dataList.stream().map(entity -> {
                RwdeditRefTypeModel model = entity.toModel();
                if (withColumns != null && withColumns.contains("infos")) {
                    model.setInfos(entity.getInfos());
                }
                return model;
            }).collect(Collectors.toList());
            result.setData(collect);
        }
        return result;
    }

    @Transactional
    public MapResponse create(RwdeditRefTypeModel param) {
        MapResponse response = new MapResponse();
        String refKey = param.getRefKey();
        if (refKey == null) {
            response.setFail("refKey is required");
            return response;
        }
        if (repository.findById(refKey).isPresent()) {
            response.setFail("refKey is exists");
            return response;
        }
        RwdeditRefType refType = RwdeditRefType.fromModel(param);
        refType.setCreateUser(UserUtils.currentUserId());
        repository.save(refType);
        response.add("refKey", refKey);
        return response;
    }

    @Transactional
    public MapResponse update(RwdeditRefTypeModel param) {
        MapResponse response = new MapResponse();
        if (param.getRefKey() == null) {
            response.setFail("refKey is required");
            return response;
        }
        Optional<RwdeditRefType> optional = repository.findById(param.getRefKey());
        if (!optional.isPresent()) {
            response.setFail("refKey is exists");
            return response;
        }
        RwdeditRefType refType = optional.get();
        if (param.getName() != null) {
            refType.setName(param.getName());
            refType.setUpdateUser(UserUtils.currentUserId());
        }
        repository.save(refType);
        response.add("refKey", refType.getRefKey());
        return response;
    }

    @Transactional
    public MapResponse delete(RwdeditRefTypeModel param) {
        MapResponse response = new MapResponse();
        String refKey = param.getRefKey();
        if (refKey == null) {
            response.setFail("refKey is required");
            return response;
        }
        repository.deleteById(param.getRefKey());
        return response;
    }
}

